==> symbol-processing-aa-embeddable-2.2.0-2.0.2-sources.jar: org/jetbrains/kotlin/resolve/checkers/KClassWithIncorrectTypeArgumentChecker.kt <==
/*
 * Copyright 2010-2017 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ksp.org.jetbrains.kotlin.resolve.checkers

import ksp.org.jetbrains.kotlin.builtins.KotlinBuiltIns
import ksp.org.jetbrains.kotlin.descriptors.*
import ksp.org.jetbrains.kotlin.diagnostics.Errors
import ksp.org.jetbrains.kotlin.psi.KtCallableDeclaration
import ksp.org.jetbrains.kotlin.psi.KtDeclaration
import ksp.org.jetbrains.kotlin.types.UnwrappedType
import ksp.org.jetbrains.kotlin.types.typeUtil.builtIns
import ksp.org.jetbrains.kotlin.types.typeUtil.contains
import ksp.org.jetbrains.kotlin.types.typeUtil.isSubtypeOf

object KClassWithIncorrectTypeArgumentChecker : DeclarationChecker {
    override fun check(declaration: KtDeclaration, descriptor: DeclarationDescriptor, context: DeclarationCheckerContext) {
        if (descriptor !is CallableMemberDescriptor || descriptor.visibility == DescriptorVisibilities.LOCAL) return

        if (declaration !is KtCallableDeclaration || declaration.typeReference != null) return

        // prevent duplicate reporting
        if (descriptor is PropertyAccessorDescriptor) return

        val returnType = descriptor.returnType ?: return

        var typeParameterWithoutNotNullableUpperBound: TypeParameterDescriptor? = null
        returnType.contains { type ->
            val kClassWithBadArgument = type.isKClassWithBadArgument()
            if (kClassWithBadArgument) {
                type.arguments.singleOrNull()?.type?.constructor?.declarationDescriptor?.let {
                    if (it is TypeParameterDescriptor && it.containingDeclaration == descriptor) {
                        typeParameterWithoutNotNullableUpperBound = it
                    }
                }
            }
            kClassWithBadArgument
        }

        if (typeParameterWithoutNotNullableUpperBound != null) {
            context.trace.report(
                Errors.KCLASS_WITH_NULLABLE_TYPE_PARAMETER_IN_SIGNATURE.on(declaration, typeParameterWithoutNotNullableUpperBound!!)
            )
        }
    }

    private fun UnwrappedType.isKClassWithBadArgument(): Boolean {
        val argumentType = arguments.singleOrNull()?.let { if (it.isStarProjection) null else it.type.unwrap() } ?: return false
        val klass = constructor.declarationDescriptor as? ClassDescriptor ?: return false

        return KotlinBuiltIns.isKClass(klass) && !argumentType.isSubtypeOf(argumentType.builtIns.anyType)
    }
}

==> symbol-processing-aa-embeddable-2.3.0-sources.jar: org/jetbrains/kotlin/resolve/checkers/KClassWithIncorrectTypeArgumentChecker.kt <==
/*
 * Copyright 2010-2017 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ksp.org.jetbrains.kotlin.resolve.checkers

import ksp.org.jetbrains.kotlin.builtins.KotlinBuiltIns
import ksp.org.jetbrains.kotlin.descriptors.*
import ksp.org.jetbrains.kotlin.diagnostics.Errors
import ksp.org.jetbrains.kotlin.psi.KtCallableDeclaration
import ksp.org.jetbrains.kotlin.psi.KtDeclaration
import ksp.org.jetbrains.kotlin.types.UnwrappedType
import ksp.org.jetbrains.kotlin.types.typeUtil.builtIns
import ksp.org.jetbrains.kotlin.types.typeUtil.contains
import ksp.org.jetbrains.kotlin.types.typeUtil.isSubtypeOf

object KClassWithIncorrectTypeArgumentChecker : DeclarationChecker {
    override fun check(declaration: KtDeclaration, descriptor: DeclarationDescriptor, context: DeclarationCheckerContext) {
        if (descriptor !is CallableMemberDescriptor || descriptor.visibility == DescriptorVisibilities.LOCAL) return

        if (declaration !is KtCallableDeclaration || declaration.typeReference != null) return

        // prevent duplicate reporting
        if (descriptor is PropertyAccessorDescriptor) return

        val returnType = descriptor.returnType ?: return

        var typeParameterWithoutNotNullableUpperBound: TypeParameterDescriptor? = null
        returnType.contains { type ->
            val kClassWithBadArgument = type.isKClassWithBadArgument()
            if (kClassWithBadArgument) {
                type.arguments.singleOrNull()?.type?.constructor?.declarationDescriptor?.let {
                    if (it is TypeParameterDescriptor && it.containingDeclaration == descriptor) {
                        typeParameterWithoutNotNullableUpperBound = it
                    }
                }
            }
            kClassWithBadArgument
        }

        if (typeParameterWithoutNotNullableUpperBound != null) {
            context.trace.report(
                Errors.KCLASS_WITH_NULLABLE_TYPE_PARAMETER_IN_SIGNATURE.on(declaration, typeParameterWithoutNotNullableUpperBound)
            )
        }
    }

    private fun UnwrappedType.isKClassWithBadArgument(): Boolean {
        val argumentType = arguments.singleOrNull()?.let { if (it.isStarProjection) null else it.type.unwrap() } ?: return false
        val klass = constructor.declarationDescriptor as? ClassDescriptor ?: return false

        return KotlinBuiltIns.isKClass(klass) && !argumentType.isSubtypeOf(argumentType.builtIns.anyType)
    }
}
